#this script is designed to predict physicochemical properties for the identified peptides from peptide offgel fractionation

# the code perform the following processes:
#1. calculated peptide isoelectric point(pI)according to modified version of Frederik Lindberg's pi script
#2. delta fraction as the difference between the experimental fraction and the calculated fraction(converted from calculated pI according the IPG strip manufacture discription)
#3. delta pI as the difference between the experimental pI(the average of pI for each fraction) and the calculated pI.
#4. the frequency of each peptide between fractions
#5. Grand Average of Hydrophobicity(GRAVY).
#6. Aliphatic Index(AI).
#7. peptide length in amino acids
#8. Amino acid composition as percentage
#9. positive and nagative amino acid percentage







$start = time;
#hydropathy values for all amino acids
my $hydropath_a = 1.800 ; my $hydropath_r = -4.500; my $hydropath_n = -3.500 ; my $hydropath_d = -3.500 ; my $hydropath_c = 2.500 ; my $hydropath_q = -3.500 ; my $hydropath_e = -3.500 ; my $hydropath_g = -0.400 ; my $hydropath_h = -3.200 ; my $hydropath_i = 4.500 ; my $hydropath_l = 3.800 ; my $hydropath_k = -3.900 ; my $hydropath_m = 1.900 ; my $hydropath_f = 2.800 ; my $hydropath_p = -1.600 ; my $hydropath_s = -0.800 ; my $hydropath_t = -0.700 ; my $hydropath_w = -0.900 ; my $hydropath_y = -1.300 ; my $hydropath_v = 4.200 ;
#basic and acidic amino acids
my $aas="ACDEFGHIKLMNPQRSTVWY";# all amino acids
my @acid=('C','D','E','Y');    # Acidic for pI
my @base=('H','K','R');        # Basic for pI
# pKa values of amino acids according to Bjellqvist et al .
# pKa
my %pka= ('C', 9.0,  'D',  4.05, 'E',  4.45, 'Y', 10.0,
       'H', 5.98, 'K', 10.0,  'R', 12.0);
# NT pKa
my %nt=  ('A', 7.59, 'E', 7.70, 'M', 7.00,
      'P', 8.36, 'S', 6.93, 'T', 6.82, 'V', 7.44);
# CT pKa
my %ct=  ('D', 4.55, 'E', 4.75);
#open file which contains peptide sequences in fasta format(the header line contains (set number_passive_2nd)and sequence line contains (peptide sequence_modification))
$file = shift || die "pep_offgel_fract_analysis.pl <file name which contains peptide list >\n";
open (IN, $file) || die "Can't open $file\n";
# create a new file which contain the output 
open (OUT,'>',"result.txt");
print OUT "set_number\tpassive_2nd\tpeptide_sequence\tredundancy\texp_fraction\tcal_fraction\tdelta_fraction\texp_pI\tcal_pI\tdelta_pI\tPep_GRAVY\tAliphatic_index\tPep_length\tA%\tC%\tD%\tE%\tF%\tG%\tH%\tI%\tK%\tL%\tM%\tN%\tP%\tQ%\tR%\tS%\tT%\tV%\tW%\tY%\tNeg_charge_aa%\tpos_charge_aa%\n";

# this script was used to convert concatenated peptide list (set number_passive_2nd_peptide sequence_modification)into fasta format, the header line contains (set number,passive_2nd,fraction number)and sequence line contains (peptide sequence_modification)
my %hTmp;
while($_ = <IN>){
	chomp $_;
	$hTmp{$_}++;
	if ($hTmp{$_}> 1) {
	}
	else{
         $_ =~ s/^(.*)\/(.*)/$1.$2/g;
         $acc = $1;
         $pep_seq =$2;
         push(@lines,"$acc\n$pep_seq\n");
	}
     #print OUT "$1\n$2\n";
}
$line = join("\n",@lines);
@lines = split("\n", $line);

#while ($line = <IN>){
foreach $line(@lines){
	chomp ($line);
	$line =~ s/\s//g;
	if ($line =~ /^[0-9]/){
	    $cou++;
	    $acc = $line."_$cou";
	    push(@accs,$acc);
	    $pep = "";
	}
	elsif ($line =~/^[A-Z]/){
	      $pep = $line;
	      #to count the frequency of each peptide between fractions
	      $hTm{$pep}++;
	      $hash{$acc} = $pep;
	      next if ($hTm{$pep} > 1);
	}
}
foreach $acc(@accs){
	chomp;
	my $ACount = 0; my $CCount = 0; my $DCount = 0; my $ECount = 0; my $FCount = 0;
	my $GCount = 0; my $HCount = 0; my $ICount = 0; my $KCount = 0;  my $LCount = 0;
	my $MCount = 0; my $NCount = 0; my $PCount = 0;  my $QCount = 0; my $RCount = 0; my $SCount = 0;  my $TCount = 0; my $VCount = 0; my $WCount = 0; my $YCount = 0;
	$pep = $hash{$acc};
	
	# to get the amino acid composition for each peptide
	     my $pep_length = length($pep);
	     for( $position= 0 ;  $position < $pep_length;  $position++){
		  $char = substr($pep,$position,1);
		  if ($char eq 'A') {
		      $ACount ++;
		  }
		  elsif ($char eq 'C') {
			 $CCount ++;
		  }
		  elsif ($char eq 'D') {
			 $DCount ++;
		  }
		  elsif ($char eq 'E') {
			 $ECount ++;
		  }
		  elsif ($char eq 'F') {
			 $FCount ++;
		  }
		  elsif ($char eq 'G') {
			 $GCount ++;
		  }
		  elsif ($char eq 'H') {
			 $HCount ++;
		  }
		  elsif ($char eq 'I') {
			 $ICount ++;
		  }
		  elsif ($char eq 'K') {
			 $KCount ++;
		 }
		  elsif ($char eq 'L') {
			 $LCount ++;
		  }
		  elsif ($char eq 'M') {
			 $MCount++;
		  }
		  elsif ($char eq 'N') {
			 $NCount ++;
		  }
		  elsif ($char eq 'P') {
			 $PCount ++;
		  }
		  elsif ($char eq 'Q') {
			 $QCount ++;
		  }
		  elsif ($char eq 'R') {
			 $RCount ++;
		  }
		  elsif ($char eq 'S') {
			$SCount ++;
		  }
		  elsif ($char eq 'T') {
			$TCount++;
		  }
		  elsif ($char eq 'V') {
			 $VCount ++;
		  }
		  elsif ($char eq 'W') {
			 $WCount ++;
		  }
		  elsif ($char eq 'Y') {
			 $YCount ++;		       
		  }
	  
	}
	# to get pI according Frederik Lindberg's pi script
	my $first = substr($pep,0,1);    # get NT aa
	my $last = substr($pep, -1,1);    # get CT aa
	my %base; my %acid;
	$base{'start'}=1;            # One amino terminus
	$acid{'end'}=1;            # One carboxy terminus
  
	# NT different for some aa, 7.50 default
	if($nt{$first}) {$pka{'start'} = $nt{$first}} else {$pka{'start'} =7.50}
  
	# CT different for some aa, 3.55 default
	if($ct{$last}) {$pka{'end'} = $ct{$last}} else {$pka{'end'} = 3.55}
  
	my %residue;
	my $cou = 0 ;
	foreach my $aa (split('', $aas)) {$residue{$aa} = ($pep =~ s/$aa//g)}
	foreach my $aa (@acid) {
		if($residue{$aa}) {
		   $acid{$aa} = $residue{$aa};    # collect acids
		   $cou++;
		}
	}
	foreach my $aa (@base) {
		if($residue{$aa}) {
		   $base{$aa} = $residue{$aa};    # collect bases
		}
	}
			# binary search for pI
	my $hi=14;                # Theoretical max
	my $lo=0;                # Theoretical min
	my $pI=7;
	my $old_pI=1;
	my $iterations = 0;
  
	while(abs($pI-$old_pI)>0.001) {    # Two correct decimals
		if($iterations++ > 15) {    # 14/0.001 &gt; 2^14, so this shouldn't happen
		   print STDERR "Excessive iterations. Something's badly wrong\n";
		   last;
	}
	my $result=0;
	foreach my $aa (keys(%acid)) { # Left (acid) side
		unless ($pka{$aa}) {print STDERR "pka for $aa NOT FOUND\n"}
                $result += $acid{$aa}/(1+10**($pka{$aa}-$pI));
	}
	foreach my $aa (keys(%base)) {    # Right (base) side
	        $result -= $base{$aa}/(1+10**($pI-$pka{$aa}));
	}
	$old_pI = $pI;
	  if($result > 0){
	    $pI=($pI+$lo)/2;        # Go lower since charge is neg
	    $hi=$old_pI;
	} else {
	    $pI=($hi+$pI)/2;        # Go higher; charge is pos
	    $lo=$old_pI;
	  }
	}
       my $rounded_pI = sprintf("%.2f", $pI);
       
       #to get the calculated fraction from each calculated pI according the IPG strip manufacture discription
       if ($rounded_pI <= 3.5) {
	    $cal_fraction = '1';
	
       }
       elsif(($rounded_pI >3.5) && ($rounded_pI <= 4.1)){
	    $cal_fraction = '2';
	   
       }
       elsif(($rounded_pI >4.1) && ($rounded_pI <= 4.7)){
	    $cal_fraction = '3';
       }
       elsif(($rounded_pI >4.7) && ($rounded_pI <= 5.3)){
	    $cal_fraction = '4';
	    
       }
       elsif(($rounded_pI >5.3) && ($rounded_pI <= 5.9)){
	    $cal_fraction = '5';
	 
       }
       elsif(($rounded_pI >5.9) && ($rounded_pI <= 6.5)){
	    $cal_fraction = '6';
	  
       }
       elsif(($rounded_pI >6.5) && ($rounded_pI <= 7.0)){
	    $cal_fraction = '7';
	   
       }
       elsif(($rounded_pI >7.0) && ($rounded_pI <= 7.6)){
	    $cal_fraction = '8';
	   
       }
       elsif(($rounded_pI >7.6) && ($rounded_pI <= 8.2)){
	    $cal_fraction = '9';
	   
       }
       elsif(($rounded_pI >8.2) && ($rounded_pI <= 8.8)){
	    $cal_fraction = '10';
	   
       }
       elsif(($rounded_pI >8.8) && ($rounded_pI <= 9.4)){
	   $cal_fraction = '11';
	   
       }
       elsif($rounded_pI >9.4) {
	   $cal_fraction = '12';
	   
       }
       
       # to extract set number, passive or 2nd recovery and fraction number from each header
	$acc_mod = $acc;
	$acc_mod =~ s/(.*)_(.*)_(.*)_/$1.$2.$3/g;
	$set = $1;
	$p_2nd = $2;
	$exp_fraction = $3;
	#to compute delta fraction
        $delta_fraction = $exp_fraction - $cal_fraction ;
        # to compute the experimental pI as the average of pI for each fraction according the IPG strip manufacture discription.
	if ($exp_fraction == 1) {
	    $exp_pI = '3';
	}
	elsif($exp_fraction == 2){
	     $exp_pI = '3.85';
	}
	elsif($exp_fraction == 3){
	     $exp_pI = '4.45';
	}
	elsif($exp_fraction == 4){
	     $exp_pI = '5.05';
	}
	elsif($exp_fraction == 5){
	     $exp_pI = '5.65';
	}
	elsif($exp_fraction == 6){
	     $exp_pI = '6.25';
	}
	elsif($exp_fraction == 7){
	     $exp_pI = '6.8';
	}
	elsif($exp_fraction == 8){
	     $exp_pI = '7.35';
	}
	elsif($exp_fraction == 9){
	     $exp_pI = '7.95';
	}
	elsif($exp_fraction == 10){
	     $exp_pI = '8.55';
	}
	elsif($exp_fraction == 11){
	     $exp_pI = '9.15';
	}
	elsif($exp_fraction == 12){
	     $exp_pI = '11.34';
	}
	#to compute delta pI
	$delta_pI = $exp_pI - $rounded_pI ;
	
       #to compute GRAVY (Grand of Hydorphopicity)for each peptide
       $GRAVY = (($hydropath_a*$ACount)+ ($hydropath_c*$CCount) + ($hydropath_d*$DCount) + ($hydropath_e*$ECount) + ($hydropath_f*$FCount) + ($hydropath_g*$GCount) + ($hydropath_h*$HCount)+ ($hydropath_i*$ICount)+ ($hydropath_k*$KCount)+ ($hydropath_l*$LCount)+($hydropath_m*$MCount)+($hydropath_n*$NCount)+ ($hydropath_p*$PCount)+ ($hydropath_q*$QCount)+ ($hydropath_r*$RCount)+ ($hydropath_s*$SCount)+ ($hydropath_t*$TCount)+ ($hydropath_v*$VCount)+ ($hydropath_w*$WCount)+($hydropath_y*$YCount))/$pep_length;
       my $rounded_GRAVY = sprintf("%.3f", $GRAVY);
	
       ###5.to compute Aliphatic index in each peptide
       my $Aliphatic_ind =((($ACount*100)/$pep_length) + (2.9 * (($VCount*100)/$pep_length)) + (3.9 * ((($ICount*100)/$pep_length) + (($LCount*100)/$pep_length)))) ;
       my $rounded_Aliphatic_ind = sprintf("%.2f", $Aliphatic_ind);
       
	#to count the negative and Positive charge aa in each peptide
       my $neg_charge_aa = $DCount + $ECount;
       my $pos_charge_aa = $RCount + $KCount; 
	
       ###6.to compute each aa percent in each peptide
       my $ACount_percent = (($ACount*100)/$pep_length);
       my $rounded_ACount_percent = sprintf("%.1f", $ACount_percent);
       my $CCount_percent = (($CCount*100)/$pep_length);
       my $rounded_CCount_percent = sprintf("%.1f", $CCount_percent);
       my $DCount_percent = (($DCount*100)/$pep_length);
       my $rounded_DCount_percent = sprintf("%.1f", $DCount_percent);
       my $ECount_percent = (($ECount*100)/$pep_length);
       my $rounded_ECount_percent = sprintf("%.1f", $ECount_percent);
       my $FCount_percent = (($FCount*100)/$pep_length);
       my $rounded_FCount_percent = sprintf("%.1f", $FCount_percent);
       my $GCount_percent = (($GCount*100)/$pep_length);
       my $rounded_GCount_percent = sprintf("%.1f", $GCount_percent);
       my $HCount_percent = (($HCount*100)/$pep_length);
       my $rounded_HCount_percent = sprintf("%.1f", $HCount_percent);
       my $ICount_percent = (($ICount*100)/$pep_length);
       my $rounded_ICount_percent = sprintf("%.1f", $ICount_percent);
       my $KCount_percent = (($KCount*100)/$pep_length);
       my $rounded_KCount_percent = sprintf("%.1f", $KCount_percent);
       my $LCount_percent = (($LCount*100)/$pep_length);
       my $rounded_LCount_percent = sprintf("%.1f", $LCount_percent);
       my $MCount_percent = (($MCount*100)/$pep_length);
       my $rounded_MCount_percent = sprintf("%.1f", $MCount_percent);
       my $NCount_percent = (($NCount*100)/$pep_length);
       my $rounded_NCount_percent = sprintf("%.1f", $NCount_percent);
       my $PCount_percent = (($PCount*100)/$pep_length);
       my $rounded_PCount_percent = sprintf("%.1f", $PCount_percent);
       my $QCount_percent = (($QCount*100)/$pep_length);
       my $rounded_QCount_percent = sprintf("%.1f", $QCount_percent);
       my $RCount_percent = (($RCount*100)/$pep_length);
       my $rounded_RCount_percent = sprintf("%.1f", $RCount_percent);
       my $SCount_percent = (($SCount*100)/$pep_length);
       my $rounded_SCount_percent = sprintf("%.1f", $SCount_percent);
       my $TCount_percent = (($TCount*100)/$pep_length);
       my $rounded_TCount_percent = sprintf("%.1f", $TCount_percent);
       my $VCount_percent = (($VCount*100)/$pep_length);
       my $rounded_VCount_percent = sprintf("%.1f", $VCount_percent);
       my $WCount_percent = (($WCount*100)/$pep_length);
       my $rounded_WCount_percent = sprintf("%.1f", $WCount_percent);
       my $YCount_percent = (($YCount*100)/$pep_length);
       my $rounded_YCount_percent = sprintf("%.1f", $YCount_percent);
       my $neg_charge_aa_percent = (($neg_charge_aa * 100)/$pep_length);
       my $rounded_neg_charge_aa_percent = sprintf("%.1f", $neg_charge_aa_percent);
       my $pos_charge_aa_percent = (($pos_charge_aa * 100)/$pep_length);
       my $rounded_pos_charge_aa_percent = sprintf("%.1f", $pos_charge_aa_percent);  
       #print (set number,passive or 2nd recovery,peptide sequence,experimental fraction,calculated fraction,delta fraction,experimentaal pI,calculted pi,delta pI,GRAVY,Aliphatic index,peptide length,amino acid composition, asidic and basic amino acid percent)
       print OUT "$set\t$p_2nd\t$hash{$acc}\t$hTm{$hash{$acc}}\t$exp_fraction\t$cal_fraction\t$delta_fraction\t$exp_pI\t$rounded_pI\t$delta_pI\t$rounded_GRAVY\t$rounded_Aliphatic_ind\t$pep_length\t$rounded_ACount_percent\t$rounded_CCount_percent\t$rounded_DCount_percent\t$rounded_ECount_percent\t$rounded_FCount_percent\t$rounded_GCount_percent\t$rounded_HCount_percent\t$rounded_ICount_percent\t$rounded_KCount_percent\t$rounded_LCount_percent\t$rounded_MCount_percent\t$rounded_NCount_percent\t$rounded_PCount_percent\t$rounded_QCount_percent\t$rounded_RCount_percent\t$rounded_SCount_percent\t$rounded_TCount_percent\t$rounded_VCount_percent\t$rounded_WCount_percent\t$rounded_YCount_percent\t$rounded_neg_charge_aa_percent\t$rounded_pos_charge_aa_percent\n";
 } 
$pro_time = time - $start;
print "processing_time : $pro_time s\n";	